@@ -9,7 +9,7 @@ use Cwd qw( abs_path );
 use Data::Compare;
 use Data::Dump;
 use Encode;
-use File::Slurp;
+use File::Slurper;
 use File::Spec;
 use Carp;
 use List::AllUtils qw(first max);
@@ -22,8 +22,8 @@ use Unicode::Normalize;
 use parent qw(Class::Accessor);
 __PACKAGE__->follow_best_practice;
 
-our $VERSION = '2.7';
-our $BETA_VERSION = 1; # Is this a beta version?
+our $VERSION = '2.9';
+our $BETA_VERSION = 0; # Is this a beta version?
 
 our $logger  = Log::Log4perl::get_logger('main');
 our $screen  = Log::Log4perl::get_logger('screen');
@@ -57,10 +57,6 @@ $CONFIG->{state}{uniqignore} = {};
 
 $CONFIG->{state}{crossrefkeys} = {};
 $CONFIG->{state}{xrefkeys} = {};
-$CONFIG->{state}{seenname} = {};
-$CONFIG->{state}{seentitle} = {};
-$CONFIG->{state}{seenbaretitle} = {};
-$CONFIG->{state}{seenwork} = {};
 
 # Set tracking, parent->child and child->parent
 $CONFIG->{state}{set}{pc} = {};
@@ -69,9 +65,6 @@ $CONFIG->{state}{set}{cp} = {};
 # Citekeys which refer to the same entry
 $CONFIG->{state}{citkey_aliases} = {};
 
-# Disambiguation data for labelalpha. Used for labelalphatemplate autoinc method
-$CONFIG->{state}{ladisambiguation} = {};
-
 # Record of which entries have inherited from other fields. Used for loop detection.
 $CONFIG->{state}{crossref} = [];
 $CONFIG->{state}{xdata} = [];
@@ -80,35 +73,6 @@ $CONFIG->{state}{xdata} = [];
 # Used for generating inheritance trees
 $CONFIG->{state}{graph} = {};
 
-# For the uniquelist feature. Records the number of times a name list occurs in all entries
-$CONFIG->{state}{uniquelistcount} = {};
-
-# Boolean to say whether uniquename/uniquelist information has changed
-# Default is true so that uniquename/uniquelist processing starts
-$CONFIG->{state}{unulchanged} = 1;
-
-# uniquenamecount holds a hash of familynames and familyname/initials
-$CONFIG->{state}{uniquenamecount} = {};
-# Same as uniquenamecount but for all names, regardless of visibility. Needed to track
-# uniquelist
-$CONFIG->{state}{uniquenamecount_all} = {};
-# Counter for tracking name/year combinations for extrayear
-$CONFIG->{state}{seen_nameyear} = {};
-# Counter for the actual extrayear value
-$CONFIG->{state}{seen_extrayear} = {};
-
-# Counter for tracking name/title combinations for extratitle
-$CONFIG->{state}{seen_nametitle} = {};
-# Counter for the actual extratitle value
-$CONFIG->{state}{seen_extratitle} = {};
-
-# Counter for tracking title/year combinations for extratitleyear
-$CONFIG->{state}{seen_titleyear} = {};
-# Counter for the actual extratitleyear value
-$CONFIG->{state}{seen_extratitleyear} = {};
-
-# Counter for the actual extraalpha value
-$CONFIG->{state}{seen_extraalpha} = {};
 $CONFIG->{state}{seenkeys} = {};
 
 # Track the order of keys as cited. Keys cited in the same \cite*{} get the same order
@@ -129,26 +93,9 @@ $CONFIG->{state}{datafiles} = [];
 
 sub _init {
   $CONFIG->{state}{uniqignore} = {};
-  $CONFIG->{options}{biblatex}{ENTRY} = {};
-  $CONFIG->{state}{unulchanged} = 1;
   $CONFIG->{state}{control_file_location} = '';
-  $CONFIG->{state}{seenname} = {};
-  $CONFIG->{state}{seentitle} = {};
-  $CONFIG->{state}{seenbaretitle} = {};
-  $CONFIG->{state}{seenwork} = {};
   $CONFIG->{state}{crossrefkeys} = {};
   $CONFIG->{state}{xrefkeys} = {};
-  $CONFIG->{state}{ladisambiguation} = {};
-  $CONFIG->{state}{uniquenamecount} = {};
-  $CONFIG->{state}{uniquenamecount_all} = {};
-  $CONFIG->{state}{uniquelistcount} = {};
-  $CONFIG->{state}{seen_nameyear} = {};
-  $CONFIG->{state}{seen_extrayear} = {};
-  $CONFIG->{state}{seen_nametitle} = {};
-  $CONFIG->{state}{seen_extratitle} = {};
-  $CONFIG->{state}{seen_titleyear} = {};
-  $CONFIG->{state}{seen_extratitleyear} = {};
-  $CONFIG->{state}{seen_extrayearalpha} = {};
   $CONFIG->{state}{seenkeys} = {};
   $CONFIG->{state}{datafiles} = [];
   $CONFIG->{state}{crossref} = [];
@@ -241,7 +188,8 @@ sub _initopts {
   }
   else {
     # Set control file name. In a conditional as @ARGV might not be set in tests
-    if (my $bcf = $ARGV[0]) {         # ARGV is ok even in a module
+    if (defined($ARGV[0])) {         # ARGV is ok even in a module
+      my $bcf = $ARGV[0];
       $bcf .= '.bcf' unless $bcf =~ m/\.bcf$/;
       Biber::Config->setoption('bcf', $bcf);
     }
@@ -372,8 +320,7 @@ sub _config_file_set {
   if (defined($conf)) {
     require XML::LibXML::Simple;
 
-    my $buf = File::Slurp::read_file($conf);
-    $buf = NFD(decode('UTF-8', $buf));# Unicode NFD boundary
+    my $buf = NFD(File::Slurper::read_text($conf));# Unicode NFD boundary
 
     $userconf = XML::LibXML::Simple::XMLin($buf,
                                            'ForceContent' => 1,
@@ -404,12 +351,14 @@ sub _config_file_set {
                                                             qr/\A(?:or)?filter\z/,
                                                             qr/\Asortexclusion\z/,
                                                             qr/\Aexclusion\z/,
+                                                            qr/\Asortingtemplate\z/,
                                                             qr/\Asort\z/,
                                                             qr/\Alabelalpha(?:name)?template\z/,
                                                             qr/\Asortitem\z/,
+                                                            qr/\Auniquenametemplate\z/,
                                                             qr/\Apresort\z/,
                                                             qr/\Aoptionscope\z/,
-                                                            qr/\Asortingnamekey\z/,
+                                                            qr/\Asortingnamekeytemplate\z/,
                                                            ],
                                            'NsStrip' => 1,
                                            'KeyAttr' => []) or
@@ -459,25 +408,7 @@ sub _config_file_set {
   while (my ($k, $v) = each $userconf->%*) {
     # Has to be an array ref and so must come before
     # the later options tests which assume hash refs
-    if (lc($k) eq 'uniquenametemplate') {
-      my $unkt;
-      my $bun;
-      foreach my $np (sort {$a->{order} <=> $b->{order}} $v->{namepart}->@*) {
-
-        # useful later in uniqueness tests
-        if ($np->{base}) {
-          push $bun->@*, $np->{content};
-        }
-
-        push $unkt->@*, {namepart => $np->{content},
-                         use => $np->{use},
-                         base => $np->{base}}
-      }
-      Biber::Config->setblxoption('uniquenametemplate', $unkt);
-    }
-    # Has to be an array ref and so must come before
-    # the later options tests which assume hash refs
-    elsif (lc($k) eq 'labelalphatemplate') {
+    if (lc($k) eq 'labelalphatemplate') {
       foreach my $t ($v->@*) {
         my $latype = $t->{type};
         if ($latype eq 'global') {
@@ -493,8 +424,8 @@ sub _config_file_set {
     }
     elsif (lc($k) eq 'labelalphanametemplate') {
       foreach my $t ($v->@*) {
+        my $lants;
         my $lant;
-        my $lantype = $t->{type};
         foreach my $np (sort {$a->{order} <=> $b->{order}} $t->{namepart}->@*) {
           push $lant->@*, {namepart           => $np->{content},
                            use                => $np->{use},
@@ -504,18 +435,25 @@ sub _config_file_set {
                            substring_width    => $np->{substring_width} };
 
         }
-
-        if ($lantype eq 'global') {
-          Biber::Config->setblxoption('labelalphanametemplate', $lant);
-        }
-        else {
-          Biber::Config->setblxoption('labelalphanametemplate', $lant, 'ENTRYTYPE', $lantype);
-        }
+        $lants->{$t->{name}} = $lant;
+        Biber::Config->setblxoption('labelalphanametemplate', $lants);
       }
     }
-    # Has to be an array ref and so must come before
-    # the later options tests which assume hash refs
-    elsif (lc($k) eq 'sortingnamekey') {
+    elsif (lc($k) eq 'uniquenametemplate') {
+      my $unts;
+      foreach my $unt ($v->@*) {
+        my $untval = [];
+        foreach my $np (sort {$a->{order} <=> $b->{order}} $unt->{namepart}->@*) {
+          push $untval->@*, {namepart        => $np->{content},
+                             use             => $np->{use},
+                             disambiguation  => $np->{disambiguation},
+                             base            => $np->{base}};
+        }
+        $unts->{$unt->{name}} = $untval;
+      }
+      Biber::Config->setblxoption('uniquenametemplate', $unts);
+    }
+    elsif (lc($k) eq 'sortingnamekeytemplate') {
       my $snss;
       foreach my $sns ($v->@*) {
         my $snkps;
@@ -539,9 +477,9 @@ sub _config_file_set {
           }
           push $snkps->@*, $snps;
         }
-        $snss->{$sns->{keyscheme}} = $snkps;
+        $snss->{$sns->{name}} = $snkps;
       }
-      Biber::Config->setblxoption('sortingnamekey', $snss);
+      Biber::Config->setblxoption('sortingnamekeytemplate', $snss);
     }
     elsif (lc($k) eq 'transliteration') {
       foreach my $tr ($v->@*) {
@@ -591,9 +529,8 @@ sub _config_file_set {
     elsif (lc($k) eq 'inheritance') {# This is a biblatex option
       Biber::Config->setblxoption($k, $v);
     }
-    elsif (lc($k) eq 'sorting') {# This is a biblatex option
-      # sorting excludes
-      foreach my $sex ($v->{sortexclusion}->@*) {
+    elsif (lc($k) eq 'sortexclusion') {# This is a biblatex option
+      foreach my $sex ($v->@*) {
         my $excludes;
         foreach my $ex ($sex->{exclusion}->@*) {
           $excludes->{$ex->{content}} = 1;
@@ -603,9 +540,22 @@ sub _config_file_set {
                                     'ENTRYTYPE',
                                     $sex->{type});
       }
-
+    }
+    elsif (lc($k) eq 'sortinclusion') {# This is a biblatex option
+      foreach my $sin ($v->@*) {
+        my $includes;
+        foreach my $in ($sin->{inclusion}->@*) {
+          $includes->{$in->{content}} = 1;
+        }
+        Biber::Config->setblxoption('sortinclusion',
+                                    $includes,
+                                    'ENTRYTYPE',
+                                    $sin->{type});
+      }
+    }
+    elsif (lc($k) eq 'presort') {# This is a biblatex option
       # presort defaults
-      foreach my $presort ($v->{presort}->@*) {
+      foreach my $presort ($v->@*) {
         # Global presort default
         unless (exists($presort->{type})) {
           Biber::Config->setblxoption('presort', $presort->{content});
@@ -618,7 +568,13 @@ sub _config_file_set {
                                       $presort->{type});
         }
       }
-      Biber::Config->setblxoption('sorting', Biber::_parse_sort($v));
+    }
+    elsif (lc($k) eq 'sortingtemplate') {# This is a biblatex option
+      my $sorttemplates;
+      foreach my $ss ($v->@*) {
+        $sorttemplates->{$ss->{name}} = Biber::_parse_sort($ss);
+      }
+      Biber::Config->setblxoption('sortingtemplate', $sorttemplates);
     }
     elsif (lc($k) eq 'datamodel') {# This is a biblatex option
       Biber::Config->setblxoption('datamodel', $v);
@@ -699,29 +655,6 @@ sub config_file {
 ##############################
 
 
-=head2 get_unul_done
-
-    Return a boolean saying whether uniquenename+uniquelist processing is finished
-
-=cut
-
-sub get_unul_done {
-  shift; # class method so don't care about class name
-  return $CONFIG->{state}{unulchanged} ? 0 : 1;
-}
-
-=head2 set_unul_changed
-
-    Set a boolean saying whether uniquename+uniquelist has changed
-
-=cut
-
-sub set_unul_changed {
-  shift; # class method so don't care about class name
-  my $val = shift;
-  $CONFIG->{state}{unulchanged} = $val;
-  return;
-}
 
 =head2 add_uniq_ignore
 
@@ -1216,36 +1149,6 @@ sub is_inheritance_path {
   return 0;
 }
 
-
-=head1 labelalpha disambiguation
-
-=head2 incr_la_disambiguation
-
-    Increment a counter to say we have seen this labelalpha
-
-=cut
-
-sub incr_la_disambiguation {
-  shift; # class method so don't care about class name
-  my $la = shift;
-  $CONFIG->{state}{ladisambiguation}{$la}++;
-  return;
-}
-
-
-=head2 get_la_disambiguation
-
-    Get the disambiguation counter for this labelalpha.
-    Return a 0 for undefs to avoid spurious errors.
-
-=cut
-
-sub get_la_disambiguation {
-  shift; # class method so don't care about class name
-  my $la = shift;
-  return $CONFIG->{state}{ladisambiguation}{$la} // 0;
-}
-
 =head1 keyorder
 
 =head2 set_keyorder
@@ -1339,612 +1242,6 @@ sub incr_seenkey {
   return;
 }
 
-=head2 get_seenname
-
-    Get the count of occurrences of a labelname or labeltitle
-
-=cut
-
-sub get_seenname {
-  shift; # class method so don't care about class name
-  my $identifier = shift;
-  return $CONFIG->{state}{seenname}{$identifier};
-}
-
-=head2 incr_seenname
-
-    Increment the count of occurrences of a labelname or labeltitle
-
-=cut
-
-sub incr_seenname {
-  shift; # class method so don't care about class name
-  my $identifier = shift;
-  $CONFIG->{state}{seenname}{$identifier}++;
-  return;
-}
-
-=head2 get_seentitle
-
-    Get the count of occurrences of a labeltitle
-
-=cut
-
-sub get_seentitle {
-  shift; # class method so don't care about class name
-  my $identifier = shift;
-  return $CONFIG->{state}{seentitle}{$identifier};
-}
-
-=head2 incr_seentitle
-
-    Increment the count of occurrences of a labeltitle
-
-=cut
-
-sub incr_seentitle {
-  shift; # class method so don't care about class name
-  my $identifier = shift;
-  $CONFIG->{state}{seentitle}{$identifier}++;
-  return;
-}
-
-=head2 get_seenbaretitle
-
-    Get the count of occurrences of a labeltitle when there is
-    no labelname
-
-=cut
-
-sub get_seenbaretitle {
-  shift; # class method so don't care about class name
-  my $identifier = shift;
-  return $CONFIG->{state}{seenbaretitle}{$identifier};
-}
-
-=head2 incr_seenbaretitle
-
-    Increment the count of occurrences of a labeltitle
-    when there is no labelname
-
-=cut
-
-sub incr_seenbaretitle {
-  shift; # class method so don't care about class name
-  my $identifier = shift;
-  $CONFIG->{state}{seenbaretitle}{$identifier}++;
-  return;
-}
-
-=head2 get_seenwork
-
-    Get the count of occurrences of a labelname and labeltitle
-
-=cut
-
-sub get_seenwork {
-  shift; # class method so don't care about class name
-  my $identifier = shift;
-  return $CONFIG->{state}{seenwork}{$identifier};
-}
-
-=head2 incr_seenwork
-
-    Increment the count of occurrences of a labelname and labeltitle
-
-=cut
-
-sub incr_seenwork {
-  shift; # class method so don't care about class name
-  my $identifier = shift;
-  $CONFIG->{state}{seenwork}{$identifier}++;
-  return;
-}
-
-=head2 incr_seenpa
-
-    Increment the count of occurrences of a primary author family name
-
-=cut
-
-sub incr_seenpa {
-  shift; # class method so don't care about class name
-  my $identifier = shift;
-  $CONFIG->{state}{seenpa}{$identifier}++;
-  return;
-}
-
-
-=head2 get_seenpa
-
-    Get the count of occurrences of a primary author family name
-
-=cut
-
-sub get_seenpa {
-  shift; # class method so don't care about class name
-  my $identifier = shift;
-  return $CONFIG->{state}{seenpa}{$identifier};
-}
-
-=head2 reset_seen_extra
-
-    Reset the counters for extra*
-
-=cut
-
-sub reset_seen_extra {
-  shift; # class method so don't care about class name
-  my $ay = shift;
-  $CONFIG->{state}{seen_extrayear} = {};
-  $CONFIG->{state}{seen_extratitle} = {};
-  $CONFIG->{state}{seen_extratitleyear} = {};
-  $CONFIG->{state}{seen_extraalpha} = {};
-  return;
-}
-
-
-=head2 incr_seen_extrayear
-
-    Increment and return the counter for extrayear
-
-=cut
-
-sub incr_seen_extrayear {
-  shift; # class method so don't care about class name
-  my $ey = shift;
-  return ++$CONFIG->{state}{seen_extrayear}{$ey};
-}
-
-=head2 incr_seen_extratitle
-
-    Increment and return the counter for extratitle
-
-=cut
-
-sub incr_seen_extratitle {
-  shift; # class method so don't care about class name
-  my $et = shift;
-  return ++$CONFIG->{state}{seen_extratitle}{$et};
-}
-
-=head2 incr_seen_extratitleyear
-
-    Increment and return the counter for extratitleyear
-
-=cut
-
-sub incr_seen_extratitleyear {
-  shift; # class method so don't care about class name
-  my $ety = shift;
-  return ++$CONFIG->{state}{seen_extratitleyear}{$ety};
-}
-
-
-=head2 incr_seen_extraalpha
-
-    Increment and return the counter for extraalpha
-
-=cut
-
-sub incr_seen_extraalpha {
-  shift; # class method so don't care about class name
-  my $ea = shift;
-  return ++$CONFIG->{state}{seen_extraalpha}{$ea};
-}
-
-
-=head2 get_seen_nameyear
-
-    Get the count of an labelname/labelyear combination for tracking
-    extrayear. It uses labelyear plus name as we need to disambiguate
-    entries with different labelyear (like differentiating 1984--1986 from
-    just 1984)
-
-=cut
-
-sub get_seen_nameyear {
-  shift; # class method so don't care about class name
-  my $ny = shift;
-  return $CONFIG->{state}{seen_nameyear}{$ny};
-}
-
-=head2 incr_seen_nameyear
-
-    Increment the count of an labelname/labelyear combination for extrayear
-
-    We pass in the name and year strings separately as we have to
-    be careful and only increment this counter beyond 1 if there is
-    a name component. Otherwise, extrayear gets defined for all
-    entries with no name but the same year etc.
-
-=cut
-
-sub incr_seen_nameyear {
-  shift; # class method so don't care about class name
-  my ($ns, $ys) = @_;
-  my $tmp = "$ns,$ys";
-  # We can always increment this to 1
-  unless ($CONFIG->{state}{seen_nameyear}{$tmp}) {
-    $CONFIG->{state}{seen_nameyear}{$tmp}++;
-  }
-  # But beyond that only if we have a labelname in the entry since
-  # this counter is used to create extrayear which doesn't mean anything for
-  # entries with no name
-  # We allow empty year so that we generate extrayear for the same name with no year
-  # so we can do things like "n.d.-a", "n.d.-b" etc.
-  else {
-    if ($ns) {
-      $CONFIG->{state}{seen_nameyear}{$tmp}++;
-    }
-  }
-  return;
-}
-
-
-=head2 get_seen_nametitle
-
-    Get the count of an labelname/labeltitle combination for tracking
-    extratitle.
-
-=cut
-
-sub get_seen_nametitle {
-  shift; # class method so don't care about class name
-  my $nt = shift;
-  return $CONFIG->{state}{seen_nametitle}{$nt};
-}
-
-=head2 incr_seen_nametitle
-
-    Increment the count of an labelname/labeltitle combination for extratitle
-
-    We pass in the name and year strings separately as we have to
-    be careful and only increment this counter beyond 1 if there is
-    a title component. Otherwise, extratitle gets defined for all
-    entries with no title.
-
-=cut
-
-sub incr_seen_nametitle {
-  shift; # class method so don't care about class name
-  my ($ns, $ts) = @_;
-  my $tmp = "$ns,$ts";
-  # We can always increment this to 1
-  unless ($CONFIG->{state}{seen_nametitle}{$tmp}) {
-    $CONFIG->{state}{seen_nametitle}{$tmp}++;
-  }
-  # But beyond that only if we have a labeltitle in the entry since
-  # this counter is used to create extratitle which doesn't mean anything for
-  # entries with no title
-  else {
-    if ($ts) {
-      $CONFIG->{state}{seen_nametitle}{$tmp}++;
-    }
-  }
-  return;
-}
-
-
-=head2 get_seen_titleyear
-
-    Get the count of an labeltitle/labelyear combination for tracking
-    extratitleyear
-
-=cut
-
-sub get_seen_titleyear {
-  shift; # class method so don't care about class name
-  my $ty = shift;
-  return $CONFIG->{state}{seen_titleyear}{$ty};
-}
-
-=head2 incr_seen_titleyear
-
-    Increment the count of an labeltitle/labelyear combination for extratitleyear
-
-    We pass in the title and year strings separately as we have to
-    be careful and only increment this counter beyond 1 if there is
-    a title component. Otherwise, extratitleyear gets defined for all
-    entries with no title.
-
-=cut
-
-sub incr_seen_titleyear {
-  shift; # class method so don't care about class name
-  my ($ts, $ys) = @_;
-  my $tmp = "$ts,$ys";
-  # We can always increment this to 1
-  unless ($CONFIG->{state}{seen_titleyear}{$tmp}) {
-    $CONFIG->{state}{seen_titleyear}{$tmp}++;
-  }
-  # But beyond that only if we have a labeltitle in the entry since
-  # this counter is used to create extratitleyear which doesn't mean anything for
-  # entries with no title
-  else {
-    if ($ts) {
-      $CONFIG->{state}{seen_titleyear}{$tmp}++;
-    }
-  }
-  return;
-}
-
-=head1 uniquelistcount
-
-=head2 get_uniquelistcount
-
-    Get the number of uniquelist entries for a (possibly partial) list
-
-=cut
-
-sub get_uniquelistcount {
-  shift; # class method so don't care about class name
-  my $namelist = shift;
-  return $CONFIG->{state}{uniquelistcount}{global}{join("\x{10FFFD}", $namelist->@*)};
-}
-
-=head2 add_uniquelistcount
-
-    Incremenent the count for a list part to the data for a name
-
-=cut
-
-sub add_uniquelistcount {
-  shift; # class method so don't care about class name
-  my $namelist = shift;
-  $CONFIG->{state}{uniquelistcount}{global}{join("\x{10FFFD}", $namelist->@*)}++;
-  return;
-}
-
-=head2 add_uniquelistcount_final
-
-    Incremenent the count for a complete list to the data for a name
-
-=cut
-
-sub add_uniquelistcount_final {
-  shift; # class method so don't care about class name
-  my $namelist = shift;
-  $CONFIG->{state}{uniquelistcount}{global}{final}{join("\x{10FFFD}", $namelist->@*)}++;
-  return;
-}
-
-
-=head2 add_uniquelistcount_minyear
-
-    Incremenent the count for a list and year to the data for a name
-    Used to track uniquelist = minyear
-
-=cut
-
-sub add_uniquelistcount_minyear {
-  shift; # class method so don't care about class name
-  my ($minyearnamelist, $year, $namelist) = @_;
-  # Allow year a default in case labelname is undef
-  $CONFIG->{state}{uniquelistcount}{minyear}{join("\x{10FFFD}", $minyearnamelist->@*)}{$year // '0'}{join("\x{10FFFD}", $namelist->@*)}++;
-  return;
-}
-
-=head2 get_uniquelistcount_minyear
-
-    Get the count for a list and year to the data for a name
-    Used to track uniquelist = minyear
-
-=cut
-
-sub get_uniquelistcount_minyear {
-  shift; # class method so don't care about class name
-  my ($minyearnamelist, $year) = @_;
-  return scalar keys $CONFIG->{state}{uniquelistcount}{minyear}{join("\x{10FFFD}", $minyearnamelist->@*)}{$year}->%*;
-}
-
-
-
-=head2 get_uniquelistcount_final
-
-    Get the number of uniquelist entries for a full list
-
-=cut
-
-sub get_uniquelistcount_final {
-  shift; # class method so don't care about class name
-  my $namelist = shift;
-  my $c = $CONFIG->{state}{uniquelistcount}{global}{final}{join("\x{10FFFD}", $namelist->@*)};
-  return $c // 0;
-}
-
-
-=head2 reset_uniquelistcount
-
-    Reset the count for list parts and complete lists
-
-=cut
-
-sub reset_uniquelistcount {
-  shift; # class method so don't care about class name
-  $CONFIG->{state}{uniquelistcount} = {};
-  return;
-}
-
-=head2 list_differs_nth
-
-    Returns true if some other list differs at passed nth place
-    and is at least as long
-
-    list_differs_nth([a, b, c, d, e], 3) = 1
-
-    if there is another list like any of these:
-
-    [a, b, d, e, f]
-    [a, b, e, z, z, y]
-
-=cut
-
-sub list_differs_nth {
-  shift; # class method so don't care about class name
-  my ($list, $n) = @_;
-  my @list_one = $list->@*;
-  # Loop over all final lists, looking for ones which match:
-  # * up to n - 1
-  # * differ at $n
-  # * are at least as long
-  foreach my $l_s (keys $CONFIG->{state}{uniquelistcount}{global}{final}->%*) {
-    my @l = split("\x{10FFFD}", $l_s);
-    # If list is shorter than the list we are checking, it's irrelevant
-    next if $#l < $list->$#*;
-    # If list matches at $n, it's irrelevant
-    next if ($list_one[$n-1] eq $l[$n-1]);
-    # If list doesn't match up to $n - 1, it's irrelevant
-    next unless Compare([@list_one[0 .. $n-2]], [@l[0 .. $n-2]]);
-    if ($logger->is_trace()) {# performance tune
-      $logger->trace("list_differs_nth() returning true: " . join(',', @list_one) . " vs " . join(',', @l));
-    }
-    return 1;
-  }
-  return 0;
-}
-
-=head2 list_differs_index
-
-    Returns the greatest index at where the list begins to differ from any other list
-
-    Assuming these lists
-
-    [a, b]
-    [a, b, d, e, f, g, h, i, j]
-    [a, b, d, e, f]
-    [a, b, e, z, z, y]
-
-    list_differs_index([a, b, c, d, e]) -> 2
-    list_differs_index([a]) -> 1
-
-=cut
-
-sub list_differs_index {
-  shift; # class method so don't care about class name
-  my @list = shift->@*;
-  my $index;
-  foreach my $l_s (keys $CONFIG->{state}{uniquelistcount}{global}{final}->%*) {
-    my @l = split("\x{10FFFD}", $l_s);
-    next if Compare(\@list, \@l);# Ignore identical lists
-    for (my $i=0;$i<=$#list;$i++) {
-      if ($list[$i] eq $l[$i]) {
-        if (not defined($index) or $i > $index) {
-          $index = $i;
-        }
-      }
-      else {
-        last;
-      }
-    }
-  }
-  if ($logger->is_trace()) {# performance tune
-    $logger->trace("list_differs_index() returning: " . $index+1);
-  }
-
-  if (defined($index)) { # one or more similar lists
-    if ($index == $#list) { # There is another list which is a superset, return last index
-      return $index;
-    }
-    else { # Differs with some list, return index of where difference begins
-      return $index+1;
-    }
-  }
-  else { # no similar lists
-    return undef;
-  }
-}
-
-=head1 uniquenamecount
-
-=head2 get_numofuniquenames
-
-    Get the number of uniquenames entries for a visible name
-
-=cut
-
-sub get_numofuniquenames {
-  shift; # class method so don't care about class name
-  my ($name, $namecontext) = @_;
-  my $return = scalar keys $CONFIG->{state}{uniquenamecount}{$name}{$namecontext}->%*;
-  if ($logger->is_trace()) {# performance tune
-    $logger->trace("get_numofuniquenames() returning $return for NAME='$name' and NAMECONTEXT='$namecontext'");
-  }
-  return $return;
-}
-
-=head2 get_numofuniquenames_all
-
-    Get the number of uniquenames entries for a name
-
-=cut
-
-sub get_numofuniquenames_all {
-  shift; # class method so don't care about class name
-  my ($name, $namecontext) = @_;
-  my $return = scalar keys $CONFIG->{state}{uniquenamecount_all}{$name}{$namecontext}->%*;
-  if ($logger->is_trace()) {# performance tune
-    $logger->trace("get_numofuniquenames_all() returning $return for NAME='$name' and NAMECONTEXT='$namecontext'");
-  }
-  return $return;
-}
-
-
-=head2 add_uniquenamecount
-
-    Add a name to the list of name contexts which have the name in it
-    (only called for visible names)
-
-=cut
-
-sub add_uniquenamecount {
-  shift; # class method so don't care about class name
-  my ($name, $namecontext, $key) = @_;
-  $CONFIG->{state}{uniquenamecount}{$name}{$namecontext}{$key}++;
-  return;
-}
-
-=head2 add_uniquenamecount_all
-
-    Add a name to the list of name contexts which have the name in it
-    (called for all names)
-
-=cut
-
-sub add_uniquenamecount_all {
-  shift; # class method so don't care about class name
-  my ($name, $namecontext, $key) = @_;
-  $CONFIG->{state}{uniquenamecount_all}{$name}{$namecontext}{$key}++;
-  return;
-}
-
-=head2 reset_uniquenamecount
-
-    Reset the list of names which have the name part in it
-
-=cut
-
-sub reset_uniquenamecount {
-  shift; # class method so don't care about class name
-  $CONFIG->{state}{uniquenamecount} = {};
-  $CONFIG->{state}{uniquenamecount_all} = {};
-  return;
-}
-
-=head2 _get_uniquename
-
-    Get the list of name contexts which contain a name
-    Mainly for use in tests
-
-=cut
-
-sub _get_uniquename {
-  shift; # class method so don't care about class name
-  my ($name, $namecontext) = @_;
-  my @list = sort keys $CONFIG->{state}{uniquenamecount}{$name}{$namecontext}->%*;
-  return \@list;
-}
-
 =head1 crossrefkeys
 
 =head2 get_crossrefkeys
@@ -2081,7 +1378,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2016 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2017 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
