@@ -1,39 +1,52 @@
 package Biber::Constants;
-use v5.16;
+use v5.24;
 use strict;
 use warnings;
 
+use Encode;
 use Encode::Alias;
 
-use base 'Exporter';
+use parent qw(Exporter);
+use Biber::Date::Format;
+use Text::CSV;
 
 our @EXPORT = qw{
                   $CONFIG_DEFAULT_BIBER
-                  %CONFIG_DEFAULT_BIBLATEX
-                  %CONFIG_SCOPE_BIBLATEX
-                  %CONFIG_BIBLATEX_PER_ENTRY_OPTIONS
-                  %NOSORT_TYPES
-                  %DM_DATATYPES
-                  %LOCALE_MAP
-                  %LOCALE_MAP_R
-                  %REMOTE_MAP
+                  $CONFIG_CSV_PARSER
                   $BIBER_CONF_NAME
                   $BCF_VERSION
                   $BBL_VERSION
                   $BIBER_SORT_FINAL
                   $BIBER_SORT_NULL
                   $LABEL_FINAL
+                  %CONFIG_DEFAULT_BIBLATEX
+                  %CONFIG_OPTSCOPE_BIBLATEX
+                  %CONFIG_SCOPEOPT_BIBLATEX
+                  %CONFIG_OPTTYPE_BIBLATEX
+                  %CONFIG_BIBLATEX_ENTRY_OPTIONS
+                  %CONFIG_BIBLATEX_NAMELIST_OPTIONS
+                  %CONFIG_BIBLATEX_NAME_OPTIONS
+                  %CONFIG_META_MARKERS
+                  %CONFIG_DATE_PARSERS
+                  %DATAFIELD_SETS
+                  %DM_DATATYPES
+                  %LOCALE_MAP
+                  %LOCALE_MAP_R
+                  %REMOTE_MAP
+                  %DS_EXTENSIONS
+                  %UNIQUENAME_CONTEXTS
+                  %UNIQUENAME_VALUES
               };
 
 # Version of biblatex control file which this release expects. Matched against version
 # passed in control file. Used when checking the .bcf
-our $BCF_VERSION = '2.9';
+our $BCF_VERSION = '3.4';
 # Format version of the .bbl. Used when writing the .bbl
-our $BBL_VERSION = '2.5';
+our $BBL_VERSION = '2.9';
 
 # Global flags needed for sorting
-our $BIBER_SORT_FINAL = 0;
-our $BIBER_SORT_NULL  = 0;
+our $BIBER_SORT_FINAL;
+our $BIBER_SORT_NULL;
 
 # the name of the Biber configuration file, which should be
 # either returned by kpsewhich or located at "$HOME/.$BIBER_CONF_NAME"
@@ -52,105 +65,123 @@ unless ($locale) {
   }
 }
 
-# nosort type category shortcuts
-our %NOSORT_TYPES = (
-                     type_name => {
-                                   author => 1,
-                                   afterword => 1,
-                                   annotator => 1,
-                                   bookauthor => 1,
-                                   commentator => 1,
-                                   editor => 1,
-                                   editora => 1,
-                                   editorb => 1,
-                                   editorc => 1,
-                                   foreword => 1,
-                                   holder => 1,
-                                   introduction => 1,
-                                   namea => 1,
-                                   nameb => 1,
-                                   namec => 1,
-                                   shortauthor => 1,
-                                   shorteditor => 1,
-                                   translator => 1
-                                  },
-                     type_title => {
-                                    booktitle => 1,
-                                    eventtitle => 1,
-                                    issuetitle => 1,
-                                    journaltitle => 1,
-                                    maintitle => 1,
-                                    origtitle => 1,
-                                    title => 1
-                                   }
-);
+
+# datafieldsets
+our %DATAFIELD_SETS = ();
 
 # datatypes for data model validation
 our %DM_DATATYPES = (
-                     integer => qr/\A\d+\z/xms,
+                     integer  => qr/\A\d+\z/xms,
                      datepart => qr/\A\d+\z/xms
-);
+                    );
+
+# Mapping of data source and output types to extensions
+our %DS_EXTENSIONS = (
+                      bbl        => 'bbl',
+                      bblxml     => 'bblxml',
+                      bibtex     => 'bib',
+                      biblatexml => 'bltxml'
+                      );
+
+# Mapping of biblatex uniquename option to disambiguation level
+our %UNIQUENAME_CONTEXTS = (0 => 'none',        # false
+                            1 => 'init',        # init
+                            2 => 'initorfull',  # full/true
+                            3 => 'init',        # allinit
+                            4 => 'initorfull',  # allfull
+                            5 => 'init',        # mininit
+                            6 => 'initorfull'); # minfull
+
+# Mapping of strings to numeric uniquename values for easier biblatex processing
+our %UNIQUENAME_VALUES = ('none' => 0, 'init' => 1, full => '2');
 
 # Biber option defaults. Mostly not needed outside of tool mode since they are passed by .bcf
-
 our $CONFIG_DEFAULT_BIBER = {
-  clrmacros           => { content => 0 },
-  collate             => { content => 1 },
-  collate_options     => { option => {level => 4, variable => 'non-ignorable', normalization => 'prenormalized' }},
-  graph               => { content => 0 },
-  debug               => { content => 0 },
-  decodecharsset      => { content => 'base' },
-  dot_include         => { option => {section => 1, xdata => 1, crossref => 1, xref => 1 }},
-  fastsort            => { content => 0 },
-  fixinits            => { content => 0 },
-  input_encoding      => { content => 'UTF-8' },
-  input_format        => { content => 'bibtex' },
-  isbn10              => { content => 0 },
-  isbn13              => { content => 0 },
-  isbn_normalise      => { content => 0 },
-  listsep             => { content => 'and' },
-  mincrossrefs        => { content => 2 },
-  namesep             => { content => 'and' },
-  nodieonerror        => { content => 0 },
-  noinit              => { option => [ {value => q/\b\p{Ll}{2}\p{Pd}/},
-                                       {value => q/[\x{2bf}\x{2018}]/} ] },
-  nolabel             => { option => [ {value => q/[\p{P}\p{S}\p{C}]+/} ] },
-  nolabelwidthcount   => { option => [ {value => q//} ] },
-  nolog               => { content => 0 },
-  nostdmacros         => { content => 0 },
-  nosort              => { option => [ { name => 'type_name', value => q/\A\p{L}{2}\p{Pd}/ },
-                                       { name => 'type_name', value => q/[\x{2bf}\x{2018}]/ } ] },
-  onlylog             => { content => 0 },
-  others_string       => { content => 'others' },
-  output_align        => { content => 0 },
-  output_encoding     => { content => 'UTF-8' },
-  output_format       => { content => 'bbl' },
-  output_indent       => { content => '2' },
-  output_fieldcase    => { content => 'upper' },
-  output_resolve      => { content => 0 },
-  output_safechars    => { content => 0 },
-  output_safecharsset => { content => 'base' },
-  quiet               => { content => 0 },
-  noskipduplicates    => { content => 0 },
-  sortcase            => { content => 1 },
-  sortfirstinits      => { content => 0 },
-  sortupper           => { content => 1 },
-  strip_comments      => { content => 0 },
-  tool                => { content => 0 },
-  trace               => { content => 0 },
-  validate_config     => { content => 0 },
-  validate_control    => { content => 0 },
-  validate_datamodel  => { content => 0 },
-  wraplines           => { content => 0 },
-  xsvsep              => { content => q/\s*,\s*/ },
+  annotation_marker                           => { content => q/+an/ },
+  clrmacros                                   => { content => 0 },
+  collate_options                             => { option => {level => 4, variable => 'non-ignorable', normalization => 'prenormalized' }},
+  graph                                       => { content => 0 },
+  debug                                       => { content => 0 },
+  dieondatamodel                              => { content => 0 },
+  decodecharsset                              => { content => 'base' },
+  dot_include                                 => { option => {section => 1, xdata => 1, crossref => 1, xref => 1 }},
+  fixinits                                    => { content => 0 },
+  input_encoding                              => { content => 'UTF-8' },
+  input_format                                => { content => 'bibtex' },
+  isbn10                                      => { content => 0 },
+  isbn13                                      => { content => 0 },
+  isbn_normalise                              => { content => 0 },
+  listsep                                     => { content => 'and' },
+  mincrossrefs                                => { content => 2 },
+  minxrefs                                    => { content => 2 },
+  namesep                                     => { content => 'and' },
+  no_bblxml_schema                            => { content => 0 },
+  no_bltxml_schema                            => { content => 0 },
+  nodieonerror                                => { content => 0 },
+  noinit                                      => { option => [ {value => q/\b\p{Ll}{2}\p{Pd}/},
+                                                               {value => q/[\x{2bf}\x{2018}]/} ] },
+  nolabel                                     => { option => [ {value => q/[\p{P}\p{S}\p{C}]+/} ] },
+#  nolabelwidthcount                          => { option =>  }, # default is nothing
+  nolog                                       => { content => 0 },
+  nostdmacros                                 => { content => 0 },
+  nosort                                      => { option => [ { name => 'setnames', value => q/\A\p{L}{2}\p{Pd}/ },
+                                                               { name => 'setnames', value => q/[\x{2bf}\x{2018}]/ } ] },
+  onlylog                                     => { content => 0 },
+  others_string                               => { content => 'others' },
+  output_align                                => { content => 0 },
+  output_annotation_marker                    => { content => '+an' },
+  output_encoding                             => { content => 'UTF-8' },
+  output_field_order                          => { content => 'options,abstract,names,lists,dates' },
+  output_format                               => { content => 'bbl' },
+  output_indent                               => { content => '2' },
+  output_fieldcase                            => { content => 'upper' },
+  output_listsep                              => { content => 'and' },
+  output_namesep                              => { content => 'and' },
+  output_resolve_xdata                        => { content => 0 },
+  output_resolve_crossrefs                    => { content => 0 },
+  output_resolve_sets                         => { content => 0 },
+  output_safechars                            => { content => 0 },
+  output_safecharsset                         => { content => 'base' },
+  output_xnamesep                             => { content => '=' },
+  quiet                                       => { content => 0 },
+  noskipduplicates                            => { content => 0 },
+  sortdebug                                   => { content => 0 },
+  sortcase                                    => { content => 1 },
+  sortupper                                   => { content => 1 },
+  strip_comments                              => { content => 0 },
+  tool                                        => { content => 0 },
+  trace                                       => { content => 0 },
+  nouri_encode                                => { content => 0 },
+  validate_bblxml                             => { content => 0 },
+  validate_bltxml                             => { content => 0 },
+  validate_config                             => { content => 0 },
+  validate_control                            => { content => 0 },
+  validate_datamodel                          => { content => 0 },
+  wraplines                                   => { content => 0 },
+  xnamesep                                    => { content => '=' },
+  xsvsep                                      => { content => q/\s*,\s*/ },
 };
 
+# Set up some re-usable CSV parsers here for efficiency reasons
+our $CONFIG_CSV_PARSER = Text::CSV->new ( { binary           => 1,
+                                            allow_whitespace => 1,
+                                            always_quote     => 1  } );
+
+# Set up some re-usable Date parsers here for efficiency reasons
+# We need two as the missing component data is in these objects, not
+# in the DT objects returned by ->parse_datetime() and this data will
+# likely be different for start/end
+our %CONFIG_DATE_PARSERS = ('start' => Biber::Date::Format->new(),
+                            'end'   => Biber::Date::Format->new());
+
+our %CONFIG_META_MARKERS = ();
+
 # default global options for biblatex
 # Used to set:
 # * Some tool-mode defaults (as there is no .bcf and some biblatex options
 #   cannot be set in a biber config file)
 our %CONFIG_DEFAULT_BIBLATEX = (
-                                sortscheme    => 'none',
+                                sortingtemplatename    => 'tool',
                                 useauthor     => 1,
                                 useeditor     => 1,
                                 usetranslator => 1,
@@ -238,7 +269,7 @@ our %LOCALE_MAP = (
                    'japanese'        => 'ja-JP',
                    'kannada'         => 'kn-IN',
                    'lao'             => 'lo-LA',
-                   'latin'           => 'sr-Latn',
+                   'latin'           => 'la-Latn',
                    'latvian'         => 'lv-LV',
                    'lithuanian'      => 'lt-LT',
                    'lowersorbian'    => 'dsb-DE',
@@ -269,7 +300,7 @@ our %LOCALE_MAP = (
                    'samin'           => 'se-NO',
                    'sanskrit'        => 'sa-IN',
                    'scottish'        => 'gd-GB',
-                   'serbian'         => 'sr-Cyrl',
+                   'serbian'         => 'sr-Latn',
                    'serbianc'        => 'sr-Cyrl',
                    'slovak'          => 'sk-SK',
                    'slovene'         => 'sl-SI',
@@ -464,40 +495,78 @@ our %LOCALE_MAP_R = (
                      'vi-VN'      => 'vietnamese',
                     );
 
-# Holds the scope of each of the BibLaTeX configuration options fro the .bcf
-our %CONFIG_SCOPE_BIBLATEX;
+# Holds the scope of each of the BibLaTeX configuration options from the .bcf
+our %CONFIG_OPTSCOPE_BIBLATEX;
+# Holds the options in a particular scope
+our %CONFIG_SCOPEOPT_BIBLATEX;
+# Holds the datatype of an option at a particular scope
+our %CONFIG_OPTTYPE_BIBLATEX;
 
 # For per-entry options, what should be set when we find them and
 # what should be output to the .bbl for biblatex.
 # Basically, here we have to emulate relevant parts of biblatex's options processing
 # for local entry-specific options, note therefore the presence here of some
 # options like max/mincitenames which are not passed in the .bcf
-our %CONFIG_BIBLATEX_PER_ENTRY_OPTIONS =  (
-  dataonly       => {OUTPUT => 1, INPUT => {'skiplab' => 1,
-                                            'skipbiblist' => 1,
-                                            'uniquename' => 0,
-                                            'uniquelist' => 0}},
-  maxitems       => {OUTPUT => 1},
-  minitems       => {OUTPUT => 1},
-  maxbibnames    => {OUTPUT => 1},
-  minbibnames    => {OUTPUT => 1},
-  maxcitenames   => {OUTPUT => 1},
-  mincitenames   => {OUTPUT => 1},
-  maxalphanames  => {OUTPUT => 0},
-  minalphanames  => {OUTPUT => 0},
-  maxnames       => {OUTPUT => ['maxcitenames', 'maxbibnames'], INPUT  => ['maxcitenames', 'maxbibnames']},
-  minnames       => {OUTPUT => ['mincitenames', 'minbibnames'], INPUT  => ['mincitenames', 'minbibnames']},
-  presort        => {OUTPUT => 0},
-  skipbib        => {OUTPUT => 1},
-  skipbiblist    => {OUTPUT => 1},
-  skiplab        => {OUTPUT => 1},
-  uniquelist     => {OUTPUT => 0},
-  useauthor      => {OUTPUT => 1},
-  useeditor      => {OUTPUT => 1},
-  useprefix      => {OUTPUT => 1},
-  usetranslator  => {OUTPUT => 1},
-);
+our %CONFIG_BIBLATEX_ENTRY_OPTIONS =
+  (
+   dataonly          => {OUTPUT => 1,
+                         INPUT => {skiplab     => 1,
+                                   skipbiblist => 1,
+                                   uniquename  => 0,
+                                   uniquelist  => 0}
+                        },
+   maxitems          => {OUTPUT => 1},
+   minitems          => {OUTPUT => 1},
+   maxbibnames       => {OUTPUT => 1},
+   minbibnames       => {OUTPUT => 1},
+   maxcitenames      => {OUTPUT => 1},
+   mincitenames      => {OUTPUT => 1},
+   maxalphanames     => {OUTPUT => 0},
+   minalphanames     => {OUTPUT => 0},
+   maxnames          => {OUTPUT => ['maxcitenames', 'maxbibnames'],
+                         INPUT  => ['maxcitenames', 'maxbibnames']},
+   minnames          => {OUTPUT => ['mincitenames', 'minbibnames'],
+                         INPUT  => ['mincitenames', 'minbibnames']},
+   nametemplates     => {OUTPUT => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename'],
+                         INPUT  => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename']},
+   noinherit         => {OUTPUT => 0},
+   presort           => {OUTPUT => 0},
+   skipbib           => {OUTPUT => 1},
+   skipbiblist       => {OUTPUT => 1},
+   skiplab           => {OUTPUT => 1},
+   sortingnamekeytemplatename => {OUTPUT => 1},
+   uniquenametemplatename => {OUTPUT => 1},
+   labelalphanametemplatename => {OUTPUT => 1},
+   uniquelist        => {OUTPUT => 0},
+   useauthor         => {OUTPUT => 1},
+   useeditor         => {OUTPUT => 1},
+   useprefix         => {OUTPUT => 1},
+   usetranslator     => {OUTPUT => 1},
+  );
 
+our %CONFIG_BIBLATEX_NAMELIST_OPTIONS =
+  (
+   nametemplates     => {OUTPUT => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename'],
+                         INPUT  => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename']},
+  );
+
+our %CONFIG_BIBLATEX_NAME_OPTIONS =
+  (
+   nametemplates     => {OUTPUT => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename'],
+                         INPUT  => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename']},
+  );
 
 1;
 
@@ -523,7 +592,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2015 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2017 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
