@@ -167,7 +167,7 @@ sub get_uniquelistcount {
 
 =head2 add_uniquelistcount
 
-    Increment the count for a list part to the data for a name
+    Incremenent the count for a list part to the data for a name
 
 =cut
 
@@ -180,7 +180,7 @@ sub add_uniquelistcount {
 
 =head2 add_uniquelistcount_final
 
-    Increment the count for a complete list to the data for a name
+    Incremenent the count for a complete list to the data for a name
 
 =cut
 
@@ -193,7 +193,7 @@ sub add_uniquelistcount_final {
 
 =head2 add_uniquelistcount_minyear
 
-    Increment the count for a list and year to the data for a name
+    Incremenent the count for a list and year to the data for a name
     Used to track uniquelist = minyear
 
 =cut
@@ -214,7 +214,7 @@ sub add_uniquelistcount_minyear {
 
 sub get_uniquelistcount_minyear {
   my ($self, $minyearnamelist, $year) = @_;
-  return scalar keys $self->{state}{uniquelistcount}{minyear}{join("\x{10FFFD}", $minyearnamelist->@*)}{$year // '0'}->%*;
+  return scalar keys $self->{state}{uniquelistcount}{minyear}{join("\x{10FFFD}", $minyearnamelist->@*)}{$year}->%*;
 }
 
 =head2 get_uniquelistcount_final
@@ -686,29 +686,6 @@ sub set_visible_bib {
   return;
 }
 
-=head2 get_visible_sort
-
-  Gets sort name list visibility
-
-=cut
-
-sub get_visible_sort {
-  my ($self, $nlid) = @_;
-  return $self->{state}{namelistdata}{$nlid}{vissort};
-}
-
-=head2 set_visible_sort
-
-  Gets sort name list visibility
-
-=cut
-
-sub set_visible_sort {
-  my ($self, $nlid, $s) = @_;
-  $self->{state}{namelistdata}{$nlid}{vissort} = $s;
-  return;
-}
-
 =head2 get_visible_alpha
 
   Gets alpha name list visibility
@@ -789,12 +766,10 @@ sub set_unul_changed {
 sub reset_seen_extra {
   my $self = shift;
   $self->{state}{seen_extradate} = {};
-  $self->{state}{seen_extraname} = {};
   $self->{state}{seen_extratitle} = {};
   $self->{state}{seen_extratitleyear} = {};
   $self->{state}{seen_extraalpha} = {};
   $self->{state}{seen_namedateparts} = {};
-  $self->{state}{seen_labelname} = {};
   $self->{state}{seen_nametitle} = {};
   $self->{state}{seen_titleyear} = {};
   return;
@@ -811,28 +786,6 @@ sub incr_seen_extradate {
   return ++$self->{state}{seen_extradate}{$ey};
 }
 
-=head2 incr_seen_extraname
-
-    Increment and return the counter for extraname
-
-=cut
-
-sub incr_seen_extraname {
-  my ($self, $en) = @_;
-  return ++$self->{state}{seen_extraname}{$en};
-}
-
-=head2 incr_seen_labelname
-
-    Increment and return a counter used to track extraname
-
-=cut
-
-sub incr_seen_labelname {
-  my ($self, $ln) = @_;
-  return ++$self->{state}{seen_labelname}{$ln};
-}
-
 =head2 incr_seen_extratitle
 
     Increment and return the counter for extratitle
@@ -913,16 +866,6 @@ sub incr_seen_namedateparts {
   return;
 }
 
-=head2 get_seen_labelname
-
-    Get the count of a labelname hash for tracking extraname
-
-=cut
-
-sub get_seen_labelname {
-  my ($self, $ln) = @_;
-  return $self->{state}{seen_labelname}{$ln} // 0;
-}
 
 =head2 get_seen_nametitle
 
@@ -1467,31 +1410,6 @@ sub set_extradatedata_for_key {
   return;
 }
 
-=head2 set_extranamedata_for_key
-
-  Saves extraname field data for a key
-
-=cut
-
-sub set_extranamedata_for_key {
-  my ($self, $key, $en) = @_;
-  return unless defined($key);
-  $self->{state}{extranamedata}{$key} = $en;
-  return;
-}
-
-=head2 get_extranamedata_for_key
-
-    Gets the extraname field data for a key
-
-=cut
-
-sub get_extranamedata_for_key {
-  my ($self, $key) = @_;
-  return unless defined($key);
-  return $self->{state}{extranamedata}{$key};
-}
-
 =head2 set_extradatedata
 
     Saves extradate field data for all keys
@@ -1505,7 +1423,7 @@ sub set_extradatedata {
 }
 
 
-=head2 get_extradatedata_for_key
+=head2 get_extradatedata
 
     Gets the extradate field data for a key
 
@@ -1782,6 +1700,8 @@ sub instantiate_entry {
   my ($self, $section, $entry, $key, $format) = @_;
   my $be = $section->bibentry($key);
   my $bee = $be->get_field('entrytype');
+  my $un = Biber::Config->getblxoption('uniquename', $bee, $key);
+  my $ul = Biber::Config->getblxoption('uniquelist', $bee, $key);
 
   return '' unless $entry and $be;
 
@@ -1836,45 +1756,49 @@ sub instantiate_entry {
       $entry_string =~ s|<BDS>LABELALPHA</BDS>|$str|gxms;
     }
 
-    # uniquelist
-    foreach my $namefield ($dmh->{namelists}->@*) {
-      next unless my $nl = $be->get_field($namefield);
-      my $nlid = $nl->get_id;
-      if (defined($self->get_uniquelist($nlid))) {
-        my $str = 'ul=' . $self->get_uniquelist($nlid);
-        $entry_string =~ s|<BDS>UL-$nlid</BDS>|$str|gxms;
-      }
-      else {
-        $entry_string =~ s|<BDS>UL-$nlid</BDS>,?||gxms;
-      }
-    }
-
     # uniquename
-    foreach my $namefield ($dmh->{namelists}->@*) {
-      next unless my $nl = $be->get_field($namefield);
-      my $nlid = $nl->get_id;
-      foreach my $n ($nl->names->@*) {
-        my $nid = $n->get_id;
-        if (defined($self->get_unsummary($nlid, $nid))) {
-          my $str = 'un=' . $self->get_unsummary($nlid, $nid);
-          $entry_string =~ s|<BDS>UNS-$nid</BDS>|$str|gxms;
-          $str = 'uniquepart=' . $self->get_unpart($nlid, $nid);
-          $entry_string =~ s|<BDS>UNP-$nid</BDS>|$str|gxms;
-          foreach my $np ($n->get_nameparts) {
-            if ($self->is_unbasepart($nlid, $nid, $np)) {
-              $entry_string =~ s|\s+<BDS>UNP-$np-$nid</BDS>,?||gxms;
+    if ($un) { # uniquename is expensive so skip if not requested
+      foreach my $namefield ($dmh->{namelists}->@*) {
+        next unless my $nl = $be->get_field($namefield);
+        my $nlid = $nl->get_id;
+        foreach my $n ($nl->names->@*) {
+          my $nid = $n->get_id;
+          if (defined($self->get_unsummary($nlid, $nid))) {
+            my $str = 'uniquename=' . $self->get_unsummary($nlid, $nid);
+            $entry_string =~ s|<BDS>UNS-$nid</BDS>|$str|gxms;
+            $str = 'uniquepart=' . $self->get_unpart($nlid, $nid);
+            $entry_string =~ s|<BDS>UNP-$nid</BDS>|$str|gxms;
+            foreach my $np ($n->get_nameparts) {
+              if ($self->is_unbasepart($nlid, $nid, $np)) {
+                $entry_string =~ s|\s+<BDS>UNP-$np-$nid</BDS>,?||gxms;
+              }
+              else {
+                $str = "${np}un=" . $self->get_unparts($nlid, $nid, $np);
+                $entry_string =~ s|<BDS>UNP-$np-$nid</BDS>|$str|gxms;
+              }
             }
-            else {
-              $str = "${np}un=" . $self->get_unparts($nlid, $nid, $np);
-              $entry_string =~ s|<BDS>UNP-$np-$nid</BDS>|$str|gxms;
+          }
+          else {
+            $entry_string =~ s|<BDS>UN[SP]-$nid</BDS>,?||gxms;
+            foreach my $np ($n->get_nameparts) {
+              $entry_string =~ s|\s+<BDS>UNP-$np-$nid</BDS>,?||gxms;
             }
           }
         }
+      }
+    }
+
+    # uniquelist
+    if ($ul) { # uniquelist is expensive so skip if not requested
+      foreach my $namefield ($dmh->{namelists}->@*) {
+        next unless my $nl = $be->get_field($namefield);
+        my $nlid = $nl->get_id;
+        if (defined($self->get_uniquelist($nlid))) {
+          my $str = 'uniquelist=' . $self->get_uniquelist($nlid);
+          $entry_string =~ s|<BDS>UL-$nlid</BDS>|$str|gxms;
+        }
         else {
-          $entry_string =~ s|<BDS>UN[SP]-$nid</BDS>,?||gxms;
-          foreach my $np ($n->get_nameparts) {
-            $entry_string =~ s|\s+<BDS>UNP-$np-$nid</BDS>,?||gxms;
-          }
+          $entry_string =~ s|<BDS>UL-$nlid</BDS>,?||gxms;
         }
       }
     }
@@ -1935,12 +1859,6 @@ sub instantiate_entry {
       }
     }
 
-    # extraname
-    if (my $e = $self->get_extranamedata_for_key($key)) {
-      my $str = "\\field{extraname}{$e}";
-      $entry_string =~ s|<BDS>EXTRANAME</BDS>|$str|gxms;
-    }
-
     # extradate
     if (my $e = $self->get_extradatedata_for_key($key)) {
       my $str = "\\field{extradate}{$e}";
@@ -2031,45 +1949,49 @@ sub instantiate_entry {
       $entry_string =~ s|<BDS>LABELALPHA</BDS>|$str|gxms;
     }
 
-    # uniquelist
-    foreach my $namefield ($dmh->{namelists}->@*) {
-      next unless my $nl = $be->get_field($namefield);
-      my $nlid = $nl->get_id;
-      if (defined($self->get_uniquelist($nlid))) {
-        my $str = $self->get_uniquelist($nlid);
-        $entry_string =~ s|\[BDS\]UL-$nlid\[/BDS\]|$str|gxms;
-      }
-      else {
-        $entry_string =~ s|\sul="\[BDS\]UL-$nlid\[/BDS\]"||gxms;
-      }
-    }
-
     # uniquename
-    foreach my $namefield ($dmh->{namelists}->@*) {
-      next unless my $nl = $be->get_field($namefield);
-      my $nlid = $nl->get_id;
-      foreach my $n ($nl->names->@*) {
-        my $nid = $n->get_id;
-        if (defined($self->get_unsummary($nlid, $nid))) {
-          my $str = $self->get_unsummary($nlid, $nid);
-          $entry_string =~ s|\[BDS\]UNS-$nid\[/BDS\]|$str|gxms;
-          $str = $self->get_unpart($nlid, $nid);
-          $entry_string =~ s|\[BDS\]UNP-$nid\[/BDS\]|$str|gxms;
-          foreach my $np ($n->get_nameparts) {
-            if ($self->is_unbasepart($nlid, $nid, $np)) {
-              $entry_string =~ s|\sun="\[BDS\]UNP-$np-$nid\[/BDS\]",?||gxms;
+    if ($un) { # uniquename is expensive so skip if not requested
+      foreach my $namefield ($dmh->{namelists}->@*) {
+        next unless my $nl = $be->get_field($namefield);
+        my $nlid = $nl->get_id;
+        foreach my $n ($nl->names->@*) {
+          my $nid = $n->get_id;
+          if (defined($self->get_unsummary($nlid, $nid))) {
+            my $str = $self->get_unsummary($nlid, $nid);
+            $entry_string =~ s|\[BDS\]UNS-$nid\[/BDS\]|$str|gxms;
+            $str = $self->get_unpart($nlid, $nid);
+            $entry_string =~ s|\[BDS\]UNP-$nid\[/BDS\]|$str|gxms;
+            foreach my $np ($n->get_nameparts) {
+              if ($self->is_unbasepart($nlid, $nid, $np)) {
+                $entry_string =~ s|\suniquename="\[BDS\]UNP-$np-$nid\[/BDS\]",?||gxms;
+              }
+              else {
+                $str = $self->get_unparts($nlid, $nid, $np);
+                $entry_string =~ s|\[BDS\]UNP-$np-$nid\[/BDS\]|$str|gxms;
+              }
             }
-            else {
-              $str = $self->get_unparts($nlid, $nid, $np);
-              $entry_string =~ s|\[BDS\]UNP-$np-$nid\[/BDS\]|$str|gxms;
+          }
+          else {
+            $entry_string =~ s#\s(?:uniquename|uniquepart)="\[BDS\]UN[SP]-$nid\[/BDS\]",?##gxms;
+            foreach my $np ($n->get_nameparts) {
+              $entry_string =~ s|\suniquename="\[BDS\]UNP-$np-$nid\[/BDS\]",?||gxms;
             }
           }
         }
+      }
+    }
+
+    # uniquelist
+    if ($ul) { # uniquelist is expensive so skip if not requested
+      foreach my $namefield ($dmh->{namelists}->@*) {
+        next unless my $nl = $be->get_field($namefield);
+        my $nlid = $nl->get_id;
+        if (defined($self->get_uniquelist($nlid))) {
+          my $str = $self->get_uniquelist($nlid);
+          $entry_string =~ s|\[BDS\]UL-$nlid\[/BDS\]|$str|gxms;
+        }
         else {
-          $entry_string =~ s#\s(?:un|uniquepart)="\[BDS\]UN[SP]-$nid\[/BDS\]",?##gxms;
-          foreach my $np ($n->get_nameparts) {
-            $entry_string =~ s|\sun="\[BDS\]UNP-$np-$nid\[/BDS\]",?||gxms;
-          }
+          $entry_string =~ s|\suniquelist="\[BDS\]UL-$nlid\[/BDS\]"||gxms;
         }
       }
     }
@@ -2129,12 +2051,6 @@ sub instantiate_entry {
       }
     }
 
-    # extraname
-    if (my $e = $self->get_extranamedata_for_key($key)) {
-      my $str = "<bbl:field name=\"extraname\">$e</bbl:field>";
-      $entry_string =~ s|<BDS>EXTRANAME</BDS>|$str|gxms;
-    }
-
     # extradate
     if (my $e = $self->get_extradatedata_for_key($key)) {
       my $str = "<bbl:field name=\"extradate\">$e</bbl:field>";
@@ -2266,6 +2182,7 @@ __END__
 
 =head1 AUTHORS
 
+François Charette, C<< <firmicus at ankabut.net> >>
 Philip Kime C<< <philip at kime.org.uk> >>
 
 =head1 BUGS
@@ -2275,7 +2192,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2012-2019 Philip Kime, all rights reserved.
+Copyright 2009-2017 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
