@@ -1,5 +1,5 @@
 package Biber::Output::test;
-use v5.16;
+use v5.24;
 use strict;
 use warnings;
 use parent qw(Biber::Output::base);
@@ -73,90 +73,155 @@ sub _printfield {
 =cut
 
 sub set_output_entry {
-  my $self = shift;
-  my $be = shift; # Biber::Entry object
+  my ($self, $be, $section, $dm) = @_;
   my $bee = $be->get_field('entrytype');
-  my $section = shift; # Section the entry occurs in
-  my $dm = shift; # Structure object
   my $acc = '';
   my $secnum = $section->number;
-
+  my $dmh = $dm->{helpers};
   my $key = $be->get_field('citekey');
+  my $un = Biber::Config->getblxoption('uniquename', $bee, $key);
+  my $ul = Biber::Config->getblxoption('uniquelist', $bee, $key);
 
   $acc .= "% sortstring = " . $be->get_field('sortstring') . "\n"
     if (Biber::Config->getoption('debug') || Biber::Config->getblxoption('debug'));
 
-  $acc .= "    \\entry{$key}{$bee}{" . join(',', @{filter_entry_options($be->get_field('options'))}) . "}\n";
+  $acc .= "    \\entry{$key}{$bee}{" . join(',', filter_entry_options($be->get_field('options'))->@*) . "}\n";
 
   # Generate set information
-  if ( $be->get_field('entrytype') eq 'set' ) {   # Set parents get \set entry ...
-    $acc .= "      \\set{" . join(',', @{$be->get_field('entryset')}) . "}\n";
+  if ($bee eq 'set') {   # Set parents get \set entry ...
+    $acc .= "      \\set{" . join(',', $be->get_field('entryset')->@*) . "}\n";
+
+    # Set parents need this - it is the labelalpha from the first entry
+    if ( Biber::Config->getblxoption('labelalpha', $bee) ) {
+      $acc .= "      <BDS>LABELALPHA</BDS>\n";
+      $acc .= "      <BDS>EXTRAALPHA</BDS>\n";
+    }
+
+    # This is special, we have to put a marker for sortinit{hash} and then replace this string
+    # on output as it can vary between lists
+    $acc .= "      <BDS>SORTINIT</BDS>\n";
+    $acc .= "      <BDS>SORTINITHASH</BDS>\n";
+
+    # labelprefix is list-specific. It is only defined if there is no shorthand
+    # (see biblatex documentation)
+    $acc .= "      <BDS>LABELPREFIX</BDS>\n";
+
+    # Label can be in set parents
+    if (my $lab = $be->get_field('label')) {
+      $acc .= "      \\field{label}{$lab}\n";
+    }
+
+    # Annotation can be in set parents
+    if (my $ann = $be->get_field('annotation')) {
+      $acc .= "      \\field{annotation}{$ann}\n";
+    }
+
+    # Skip everything else
+    # labelnumber/labelprefix etc. are generated by biblatex after reading the .bbl
+    goto ENDENTRY;
+
   }
   else { # Everything else that isn't a set parent ...
     if (my $es = $be->get_field('entryset')) { # ... gets a \inset if it's a set member
-      $acc .= "      \\inset{" . join(',', @$es) . "}\n";
+      $acc .= "      \\inset{" . join(',', $es->@*) . "}\n";
     }
   }
 
 
   # Output name fields
-  foreach my $namefield (@{$dm->get_fields_of_type('list', 'name')}) {
+  foreach my $namefield ($dm->get_fields_of_type('list', 'name')->@*) {
     next if $dm->field_is_skipout($namefield);
     if ( my $nf = $be->get_field($namefield) ) {
-      my $plo = '';
+      my $nlid = $nf->get_id;
 
       # Did we have "and others" in the data?
       if ( $nf->get_morenames ) {
         $acc .= "      \\true{more$namefield}\n";
+        # Is this name labelname? If so, provide \morelabelname
+        if (my $lni = $be->get_labelname_info) {
+          if ( $lni eq $namefield ) {
+            $acc .= "      \\true{morelabelname}\n";
+          }
+        }
       }
 
       my $total = $nf->count_names;
 
       # Add per-list options, if any
       my $lni = $be->get_labelname_info;
+
+      my $nfv = '';
+
       if (defined($lni) and
           $lni eq $namefield) {
         # Add uniquelist, if defined
         my @plo;
-        if (my $ul = $nf->get_uniquelist){
-          push @plo, "uniquelist=$ul";
+
+        # Add uniquelist if requested
+        if ($ul) {
+          push @plo, "<BDS>UL-${nlid}</BDS>";
         }
-        $plo = join(',', @plo);
+
+        # Add per-namelist options
+        foreach my $nlo (keys $CONFIG_SCOPEOPT_BIBLATEX{NAMELIST}->%*) {
+          if (defined($nf->${\"get_$nlo"})) {
+            my $nlov = $nf->${\"get_$nlo"};
+
+            if ($CONFIG_OPTTYPE_BIBLATEX{lc($nlo)} and
+                $CONFIG_OPTTYPE_BIBLATEX{lc($nlo)} eq 'boolean') {
+              $nlov = map_boolean($nlov, 'tostring');
+            }
+
+            my $oo = expand_option($nlo, $nlov, $CONFIG_BIBLATEX_NAMELIST_OPTIONS{$nlo}->{INPUT});
+            foreach my $o ($oo->@*) {
+              push @plo, $o->[0] . '=' . $o->[1];
+            }
+          }
+        }
+        $nfv = join(',', @plo);
       }
-      $acc .= "      \\name{$namefield}{$total}{$plo}{%\n";
-      foreach my $n (@{$nf->names}) {
-        $acc .= $n->name_to_bbl;
+      $acc .= "      \\name{$namefield}{$total}{$nfv}{%\n";
+      foreach my $n ($nf->names->@*) {
+        $acc .= $n->name_to_bbl($un);
       }
       $acc .= "      }\n";
     }
   }
 
-  foreach my $listfield (@{$dm->get_fields_of_fieldtype('list')}) {
+  foreach my $listfield ($dm->get_fields_of_fieldtype('list')->@*) {
     next if $dm->field_is_datatype('name', $listfield); # name is a special list
     if ( my $lf = $be->get_field($listfield) ) {
       if ( lc($be->get_field($listfield)->[-1]) eq Biber::Config->getoption('others_string') ) {
         $acc .= "      \\true{more$listfield}\n";
-        pop @$lf; # remove the last element in the array
+        pop $lf->@*; # remove the last element in the array
       };
-      my $total = $#$lf + 1;
+      my $total = $lf->$#* + 1;
       $acc .= "      \\list{$listfield}{$total}{%\n";
-      foreach my $f (@$lf) {
+      foreach my $f ($lf->@*) {
         $acc .= "        {$f}%\n";
       }
       $acc .= "      }\n";
     }
   }
 
-  my $namehash = $be->get_field('namehash');
-  $acc .= "      \\strng{namehash}{$namehash}\n" if $namehash;
+  # Output labelname hashes
+  $acc .= "      <BDS>NAMEHASH</BDS>\n";
   my $fullhash = $be->get_field('fullhash');
   $acc .= "      \\strng{fullhash}{$fullhash}\n" if $fullhash;
+  $acc .= "      <BDS>BIBNAMEHASH</BDS>\n";
+
+  # Output namelist hashes
+  foreach my $namefield ($dmh->{namelists}->@*) {
+    next unless $be->get_field($namefield);
+    $acc .= "      <BDS>${namefield}BIBNAMEHASH</BDS>\n";
+    $acc .= "      <BDS>${namefield}NAMEHASH</BDS>\n";
+    if (my $fullhash = $be->get_field("${namefield}fullhash")) {
+      $acc .= "      \\strng{${namefield}fullhash}{$fullhash}\n";
+    }
+  }
 
   if ( Biber::Config->getblxoption('labelalpha', $be->get_field('entrytype')) ) {
-    # Might not have been set due to skiplab/dataonly
-    if (my $label = $be->get_field('labelalpha')) {
-      $acc .= "      \\field{labelalpha}{$label}\n";
-    }
+    $acc .= "      <BDS>LABELALPHA</BDS>\n";
   }
 
   # This is special, we have to put a marker for sortinit{hash} and then replace this string
@@ -164,75 +229,44 @@ sub set_output_entry {
   $acc .= "      <BDS>SORTINIT</BDS>\n";
   $acc .= "      <BDS>SORTINITHASH</BDS>\n";
 
-  # The labeldate option determines whether "extrayear" is output
-  # Skip generating extrayear for entries with "skiplab" set
-  if ( Biber::Config->getblxoption('labeldate', $be->get_field('entrytype'))) {
+  # The labeldateparts option determines whether "extradate" is output
+  # Skip generating extradate for entries with "skiplab" set
+  if ( Biber::Config->getblxoption('labeldateparts', $be->get_field('entrytype'))) {
     # Might not have been set due to skiplab/dataonly
-    if (my $ey = $be->get_field('extrayear')) {
-      my $nameyear_extra = $be->get_field('nameyear_extra');
-      if ( Biber::Config->get_seen_nameyear_extra($nameyear_extra) > 1) {
-        $acc .= "      <BDS>EXTRAYEAR</BDS>\n";
-      }
+    if (my $ey = $be->get_field('extradate')) {
+      $acc .= "      <BDS>EXTRADATE</BDS>\n";
     }
-    if (my $ly = $be->get_field('labelyear')) {
-      $acc .= "      \\field{labelyear}{$ly}\n";
-    }
-    if (my $lm = $be->get_field('labelmonth')) {
-      $acc .= "      \\field{labelmonth}{$lm}\n";
-    }
-    if (my $ld = $be->get_field('labelday')) {
-      $acc .= "      \\field{labelday}{$ld}\n";
-    }
-    if ($be->field_exists('datelabelsource')) {
-      $acc .= "      \\field{datelabelsource}{" . $be->get_field('datelabelsource') .  "}\n";
+    if ($be->field_exists('labeldatesource')) {
+      $acc .= "      \\field{labeldatesource}{" . $be->get_field('labeldatesource') .  "}\n";
     }
   }
 
+  # labelprefix is list-specific. It is only defined if there is no shorthand
+  # (see biblatex documentation)
+  unless ($be->get_field('shorthand')) {
+    $acc .= "      <BDS>LABELPREFIX</BDS>\n";
+  }
+
   # The labeltitle option determines whether "extratitle" is output
   if ( Biber::Config->getblxoption('labeltitle', $bee)) {
-    # Might not have been set due to skiplab/dataonly
-    if (my $nametitle = $be->get_field('nametitle')) {
-      if ( Biber::Config->get_seen_nametitle($nametitle) > 1) {
-        $acc .= "      <BDS>EXTRATITLE</BDS>\n";
-      }
-    }
+    $acc .= "      <BDS>EXTRATITLE</BDS>\n";
   }
 
   # The labeltitleyear option determines whether "extratitleyear" is output
   if ( Biber::Config->getblxoption('labeltitleyear', $bee)) {
-    # Might not have been set due to skiplab/dataonly
-    if (my $titleyear = $be->get_field('titleyear')) {
-      if ( Biber::Config->get_seen_titleyear($titleyear) > 1) {
-        $acc .= "      <BDS>EXTRATITLEYEAR</BDS>\n";
-      }
-    }
+    $acc .= "      <BDS>EXTRATITLEYEAR</BDS>\n";
   }
 
-
   # The labelalpha option determines whether "extraalpha" is output
-  # Skip generating extraalpha for entries with "skiplab" set
-  if ( Biber::Config->getblxoption('labelalpha', $be->get_field('entrytype'))) {
-    # Might not have been set due to skiplab/dataonly
-    if (my $ea = $be->get_field('extraalpha')) {
-      my $nameyear_extra = $be->get_field('nameyear_extra');
-      if ( Biber::Config->get_seen_nameyear_extra($nameyear_extra) > 1) {
-        $acc .= "      <BDS>EXTRAALPHA</BDS>\n";
-      }
-    }
+  if ( Biber::Config->getblxoption('labelalpha', $bee)) {
+    $acc .= "      <BDS>EXTRAALPHA</BDS>\n";
   }
 
-  if ( Biber::Config->getblxoption('labelnumber', $be->get_field('entrytype')) ) {
-    if (my $sh = $be->get_field('shorthand')) {
-      $acc .= "      \\field{labelnumber}{$sh}\n";
-    }
-    elsif (my $ln = $be->get_field('labelnumber')) {
-      $acc .= "      \\field{labelnumber}{$ln}\n";
-    }
-  }
-
-  if (defined($be->get_field('singletitle'))) {
-    $acc .= "      \\true{singletitle}\n";
-  }
+  $acc .= "      <BDS>SINGLETITLE</BDS>\n";
+  $acc .= "      <BDS>UNIQUETITLE</BDS>\n";
+  $acc .= "      <BDS>UNIQUEBARETITLE</BDS>\n";
+  $acc .= "      <BDS>UNIQUEWORK</BDS>\n";
+  $acc .= "      <BDS>UNIQUEPRIMARYAUTHOR</BDS>\n";
 
   # The source field for labelname
   if (my $lni = $be->get_labelname_info) {
@@ -244,12 +278,12 @@ sub set_output_entry {
     $acc .= "      \\field{labeltitlesource}{$lti}\n";
   }
 
-  foreach my $field (sort @{$dm->get_fields_of_type('field', 'entrykey')},
-                          @{$dm->get_fields_of_type('field', 'key')},
-                          @{$dm->get_fields_of_type('field', 'integer')},
-                          @{$dm->get_fields_of_type('field', 'datepart')},
-                          @{$dm->get_fields_of_type('field', 'literal')},
-                          @{$dm->get_fields_of_type('field', 'code')}) {
+  foreach my $field (sort $dm->get_fields_of_type('field', 'entrykey')->@*,
+                          $dm->get_fields_of_type('field', 'key')->@*,
+                          $dm->get_fields_of_type('field', 'integer')->@*,
+                          $dm->get_fields_of_type('field', 'datepart')->@*,
+                          $dm->get_fields_of_type('field', 'literal')->@*,
+                          $dm->get_fields_of_type('field', 'code')->@*) {
     next if $dm->field_is_skipout($field);
     next if $dm->get_fieldformat($field) eq 'xsv';
     if ( ($dm->field_is_nullok($field) and
@@ -270,15 +304,15 @@ sub set_output_entry {
     }
   }
 
-  foreach my $field (sort @{$dm->get_fields_of_fieldformat('xsv')}) {
+  foreach my $field (sort $dm->get_fields_of_fieldformat('xsv')->@*) {
     next if $dm->field_is_skipout($field);
     next if $dm->get_datatype($field) eq 'keyword';# This is special in .bbl
     if (my $f = $be->get_field($field)) {
-      $acc .= _printfield($be, $field, join(',', @$f) );
+      $acc .= _printfield($be, $field, join(',', $f->@*) );
     }
   }
 
-  foreach my $rfield (@{$dm->get_fields_of_datatype('range')}) {
+  foreach my $rfield ($dm->get_fields_of_datatype('range')->@*) {
     if ( my $rf = $be->get_field($rfield)) {
       $rf =~ s/[-–]+/\\bibrangedash /g;
       $acc .= "      \\field{$rfield}{$rf}\n";
@@ -286,8 +320,8 @@ sub set_output_entry {
     }
   }
 
-  foreach my $vfield ((@{$dm->get_fields_of_datatype('verbatim')},
-                       @{$dm->get_fields_of_datatype('uri')})) {
+  foreach my $vfield (($dm->get_fields_of_datatype('verbatim')->@*,
+                       $dm->get_fields_of_datatype('uri')->@*)) {
     if ( my $rf = $be->get_field($vfield) ) {
       $acc .= "      \\verb{$vfield}\n";
       $acc .= "      \\verb $rf\n    \\endverb\n";
@@ -299,11 +333,12 @@ sub set_output_entry {
 
   # Append any warnings to the entry, if any
   if (my $w = $be->get_field('warnings')) {
-    foreach my $warning (@$w) {
+    foreach my $warning ($w->@*) {
       $acc .= "      \\warn{\\item $warning}\n";
     }
   }
 
+ ENDENTRY:
   $acc .= "    \\endentry\n";
 
   # Create an index by keyname for easy retrieval
@@ -323,7 +358,7 @@ sub create_output_misc {
   my $self = shift;
 
   if (my $pa = $Biber::MASTER->get_preamble) {
-    $pa = join("%\n", @$pa);
+    $pa = join("%\n", $pa->@*);
     # Decode UTF-8 -> LaTeX macros if asked to
     if (Biber::Config->getoption('output_safechars')) {
       $pa = Biber::LaTeX::Recode::latex_encode($pa);
@@ -349,16 +384,16 @@ sub output {
   $logger->info("Writing output with encoding '" . Biber::Config->getoption('output_encoding') . "'");
   $logger->info('Converting UTF-8 to TeX macros on output to .bbl') if Biber::Config->getoption('output_safechars');
 
-  foreach my $secnum (sort keys %{$data->{ENTRIES}}) {
+  foreach my $secnum (sort keys $data->{ENTRIES}->%*) {
     my $section = $self->get_output_section($secnum);
-    foreach my $list (sort {$a->get_sortschemename cmp $b->get_sortschemename} @{$Biber::MASTER->sortlists->get_lists_for_section($secnum)}) {
+    foreach my $list (sort {$a->get_sortingtemplatename cmp $b->get_sortingtemplatename} $Biber::MASTER->datalists->get_lists_for_section($secnum)->@*) {
       next unless $list->count_keys; # skip empty lists
       my $listtype = $list->get_type;
-      foreach my $k ($list->get_keys) {
+      foreach my $k ($list->get_keys->@*) {
         my $entry = $data->{ENTRIES}{$secnum}{index}{$k};
 
         # Instantiate any dynamic, list specific entry information
-        my $entry_string = $list->instantiate_entry($entry, $k);
+        my $entry_string = $list->instantiate_entry($section, $entry, $k);
 
         # If requested to convert UTF-8 to macros ...
         if (Biber::Config->getoption('output_safechars')) {
@@ -389,7 +424,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2016 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2017 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
